import { Op } from "sequelize";
import { inject, injectable } from "tsyringe";
import { getLocationForWifi } from "../proxy/wigle";
import { WifiLocationRepository } from "../repositories/wifiLocationRepository";
import { WifiLocationHistoryService } from "./wifiLocationHistoryService";

interface UpdateWifiLocationParams {
  mac: string;
  latitude: number;
  longitude: number;
}

@injectable()
export class WifiLocationService {
  constructor(
    @inject(WifiLocationRepository) private repository: WifiLocationRepository,
    @inject(WifiLocationHistoryService)
    private wifiLocationHistory: WifiLocationHistoryService
  ) {}

  public async getAllWifiLocations() {
    return this.repository.findAll();
  }

  public async getAllWifiLocationsByAddresses(macAddresses: string[]) {
    return this.repository.findAll({
      where: { mac: { [Op.in]: macAddresses } },
    });
  }

  public async getWifiLocationByMac(mac: string) {
    let wifiLocation = await this.repository.findById(mac);

    if (wifiLocation?.longitude && wifiLocation.latitude) return wifiLocation;

    const apiLocation = await getLocationForWifi(mac);

    if (apiLocation && apiLocation.totalResults >= 0) {
      wifiLocation = await this.repository.create({
        mac,
        latitude: apiLocation.results[0].trilat,
        longitude: apiLocation.results[0].trilong,
      });

      await this.wifiLocationHistory.createWifiLocationHistory(
        wifiLocation.dataValues
      );

      return wifiLocation;
    }

    return undefined;
  }

  public async updateWifiLocation(data: UpdateWifiLocationParams) {
    return this.repository.update(data.mac, data);
  }

  public async deleteWifiLocation(id: string) {
    return this.repository.delete(id);
  }
}
